@@ -29,9 +29,9 @@ G_BEGIN_DECLS
 
 typedef struct _DviDocument DviDocument;
 
-DviDocument *dvi_document_new       (void);
+GType                 dvi_document_get_type   (void) G_GNUC_CONST;
 
-GType        dvi_document_get_type  (void) G_GNUC_CONST;
+G_MODULE_EXPORT GType register_evince_backend (GTypeModule *module);
      
 G_END_DECLS
 
