@@ -27,17 +27,20 @@
  *
  */
 
+#include <config.h>
+
+#include <string.h>
+
+#include <glib/gi18n-lib.h>
+#include <gtk/gtk.h>
+
+#include <libnautilus-extension/nautilus-extension-types.h>
+#include <libnautilus-extension/nautilus-property-page-provider.h>
+
 #include "ev-properties-view.h"
 #include "ev-backends-manager.h"
 #include "ev-document-factory.h"
 
-#include <config.h>
-#include <string.h>
-#include <glib/gi18n-lib.h>
-#include <gtk/gtklabel.h>
-#include <libnautilus-extension/nautilus-extension-types.h>
-#include <libnautilus-extension/nautilus-property-page-provider.h>
-
 static GType epp_type = 0;
 static void property_page_provider_iface_init
 	(NautilusPropertyPageProviderIface *iface);
@@ -90,7 +93,7 @@ ev_properties_get_pages (NautilusPropertyPageProvider *provider,
 	char *uri = NULL;
 	GtkWidget *page, *label;
 	NautilusPropertyPage *property_page;
-	
+
 	/* only add properties page if a single file is selected */
 	if (files == NULL || files->next != NULL)
 		goto end;
@@ -141,6 +144,7 @@ nautilus_module_initialize (GTypeModule *module)
 void
 nautilus_module_shutdown (void)
 {
+	ev_backends_manager_shutdown ();
 }
 
 void
