@@ -1,5 +1,6 @@
 /* this file is part of evince, a gnome document viewer
  *
+ *  Copyright (C) 2008 Carlos Garcia Campos <carlosgc@gnome.org>
  *  Copyright (C) 2005 Red Hat, Inc
  *
  * Evince is free software; you can redistribute it and/or modify it
@@ -20,8 +21,11 @@
 #ifndef __EV_JOBS_H__
 #define __EV_JOBS_H__
 
+#include <gio/gio.h>
 #include <gtk/gtk.h>
+
 #include "ev-document.h"
+#include "ev-selection.h"
 #include "ev-window.h"
 
 G_BEGIN_DECLS
@@ -44,6 +48,9 @@ typedef struct _EvJobFontsClass EvJobFontsClass;
 typedef struct _EvJobLoad EvJobLoad;
 typedef struct _EvJobLoadClass EvJobLoadClass;
 
+typedef struct _EvJobSave EvJobSave;
+typedef struct _EvJobSaveClass EvJobSaveClass;
+
 typedef struct _EvJobPrint EvJobPrint;
 typedef struct _EvJobPrintClass EvJobPrintClass;
 
@@ -51,6 +58,7 @@ typedef struct _EvJobPrintClass EvJobPrintClass;
 #define EV_JOB(object)		             (G_TYPE_CHECK_INSTANCE_CAST((object), EV_TYPE_JOB, EvJob))
 #define EV_JOB_CLASS(klass)	             (G_TYPE_CHECK_CLASS_CAST((klass), EV_TYPE_JOB, EvJobClass))
 #define EV_IS_JOB(object)		     (G_TYPE_CHECK_INSTANCE_TYPE((object), EV_TYPE_JOB))
+#define EV_JOB_GET_CLASS(object)             (G_TYPE_INSTANCE_GET_CLASS((object), EV_TYPE_JOB, EvJobClass))
 
 #define EV_TYPE_JOB_LINKS		     (ev_job_links_get_type())
 #define EV_JOB_LINKS(object)		     (G_TYPE_CHECK_INSTANCE_CAST((object), EV_TYPE_JOB_LINKS, EvJobLinks))
@@ -77,29 +85,49 @@ typedef struct _EvJobPrintClass EvJobPrintClass;
 #define EV_JOB_LOAD_CLASS(klass)	     (G_TYPE_CHECK_CLASS_CAST((klass), EV_TYPE_JOB_LOAD, EvJobLoadClass))
 #define EV_IS_JOB_LOAD(object)		     (G_TYPE_CHECK_INSTANCE_TYPE((object), EV_TYPE_JOB_LOAD))
 
-#define EV_TYPE_JOB_PRINT                     (ev_job_print_get_type())
-#define EV_JOB_PRINT(object)                  (G_TYPE_CHECK_INSTANCE_CAST((object), EV_TYPE_JOB_PRINT, EvJobPrint))
-#define EV_JOB_PRINT_CLASS(klass)             (G_TYPE_CHECK_CLASS_CAST((klass), EV_TYPE_JOB_PRINT, EvJobPrintClass))
-#define EV_IS_JOB_PRINT(object)               (G_TYPE_CHECK_INSTANCE_TYPE((object), EV_TYPE_JOB_PRINT))
+#define EV_TYPE_JOB_SAVE		     (ev_job_save_get_type())
+#define EV_JOB_SAVE(object)	     	     (G_TYPE_CHECK_INSTANCE_CAST((object), EV_TYPE_JOB_SAVE, EvJobSave))
+#define EV_JOB_SAVE_CLASS(klass)	     (G_TYPE_CHECK_CLASS_CAST((klass), EV_TYPE_JOB_SAVE, EvJobSaveClass))
+#define EV_IS_JOB_SAVE(object)		     (G_TYPE_CHECK_INSTANCE_TYPE((object), EV_TYPE_JOB_SAVE))
+
+#define EV_TYPE_JOB_PRINT                    (ev_job_print_get_type())
+#define EV_JOB_PRINT(object)                 (G_TYPE_CHECK_INSTANCE_CAST((object), EV_TYPE_JOB_PRINT, EvJobPrint))
+#define EV_JOB_PRINT_CLASS(klass)            (G_TYPE_CHECK_CLASS_CAST((klass), EV_TYPE_JOB_PRINT, EvJobPrintClass))
+#define EV_IS_JOB_PRINT(object)              (G_TYPE_CHECK_INSTANCE_TYPE((object), EV_TYPE_JOB_PRINT))
 
 typedef enum {
-	EV_JOB_PRIORITY_LOW,
-	EV_JOB_PRIORITY_HIGH,
-} EvJobPriority;
+	EV_JOB_RUN_THREAD,
+	EV_JOB_RUN_MAIN_LOOP
+} EvJobRunMode;
 
 struct _EvJob
 {
 	GObject parent;
+	
 	EvDocument *document;
-	gboolean finished;
-	gboolean async;
+
+	EvJobRunMode run_mode;
+
+	guint cancelled : 1;
+	guint finished : 1;
+	guint failed : 1;
+	
+	GError *error;
+	GCancellable *cancellable;
+
+	guint idle_finished_id;
+	guint idle_cancelled_id;
 };
 
 struct _EvJobClass
 {
 	GObjectClass parent_class;
 
-	void    (* finished) (EvJob *job);
+	gboolean (*run)         (EvJob *job);
+	
+	/* Signals */
+	void     (* cancelled)  (EvJob *job);
+	void     (* finished)   (EvJob *job);
 };
 
 struct _EvJobLinks
@@ -114,11 +142,25 @@ struct _EvJobLinksClass
 	EvJobClass parent_class;
 };
 
+typedef enum {
+	EV_RENDER_INCLUDE_NONE      = 0,
+	EV_RENDER_INCLUDE_LINKS     = 1 << 0,
+	EV_RENDER_INCLUDE_TEXT      = 1 << 1,
+	EV_RENDER_INCLUDE_SELECTION = 1 << 2,
+	EV_RENDER_INCLUDE_IMAGES    = 1 << 3,
+	EV_RENDER_INCLUDE_FORMS     = 1 << 4,
+	EV_RENDER_INCLUDE_ALL       = (1 << 5) - 1
+} EvRenderFlags;
+
 struct _EvJobRender
 {
 	EvJob parent;
 
-	EvRenderContext *rc;
+	gint page;
+	gint rotation;
+	gdouble scale;
+
+	EvPage *ev_page;
 	gboolean page_ready;
 	gint target_width;
 	gint target_height;
@@ -132,14 +174,11 @@ struct _EvJobRender
 	cairo_surface_t *selection;
 	GdkRegion *selection_region;
 	EvRectangle selection_points;
+	EvSelectionStyle selection_style;
 	GdkColor base;
 	GdkColor text; 
 
-	gint include_forms : 1;
-	gint include_links : 1;
-	gint include_text : 1;
-	gint include_selection : 1;
-	gint include_images : 1;
+	EvRenderFlags flags;
 };
 
 struct _EvJobRenderClass
@@ -153,7 +192,10 @@ struct _EvJobThumbnail
 {
 	EvJob parent;
 
-	EvRenderContext *rc;
+	gint page;
+	gint rotation;
+	gdouble scale;
+	
 	GdkPixbuf *thumbnail;
 };
 
@@ -171,6 +213,9 @@ struct _EvJobFonts
 struct _EvJobFontsClass
 {
         EvJobClass parent_class;
+
+	/* Signals */
+	void (* updated)  (EvJobFonts *job);
 };
 
 struct _EvJobLoad
@@ -179,8 +224,9 @@ struct _EvJobLoad
 	
 	EvLinkDest *dest;
 	EvWindowRunMode mode;
-	GError *error;
+	gchar *search_string;
 	gchar *uri;
+	gchar *password;
 };
 
 struct _EvJobLoadClass
@@ -188,11 +234,23 @@ struct _EvJobLoadClass
 	EvJobClass parent_class;
 };
 
+struct _EvJobSave
+{
+	EvJob parent;
+
+	gchar *uri;
+	gchar *document_uri;
+};
+
+struct _EvJobSaveClass
+{
+	EvJobClass parent_class;
+};
+
 struct _EvJobPrint
 {
 	EvJob parent;
 
-	GError *error;
 	const gchar *format;
 	gchar  *temp_file;
 	EvPrintRange *ranges;
@@ -213,48 +271,67 @@ struct _EvJobPrintClass
 
 /* Base job class */
 GType           ev_job_get_type           (void) G_GNUC_CONST;
-void            ev_job_finished           (EvJob          *job);
+gboolean        ev_job_run                (EvJob          *job);
+void            ev_job_cancel             (EvJob          *job);
+void            ev_job_failed             (EvJob          *job,
+					   GQuark          domain,
+					   gint            code,
+					   const gchar    *format,
+					   ...);
+void            ev_job_failed_from_error  (EvJob          *job,
+					   GError         *error);
+void            ev_job_succeeded          (EvJob          *job);
+gboolean        ev_job_is_cancelled       (EvJob          *job);
+gboolean        ev_job_is_finished        (EvJob          *job);
+gboolean        ev_job_is_failed          (EvJob          *job);
+EvJobRunMode    ev_job_get_run_mode       (EvJob          *job);
+void            ev_job_set_run_mode       (EvJob          *job,
+					   EvJobRunMode    run_mode);
 
 /* EvJobLinks */
 GType           ev_job_links_get_type     (void) G_GNUC_CONST;
 EvJob          *ev_job_links_new          (EvDocument     *document);
-void            ev_job_links_run          (EvJobLinks     *thumbnail);
 
 /* EvJobRender */
 GType           ev_job_render_get_type    (void) G_GNUC_CONST;
 EvJob          *ev_job_render_new         (EvDocument      *document,
-					   EvRenderContext *rc,
+					   gint             page,
+					   gint             rotation,
+					   gdouble          scale,
 					   gint             width,
 					   gint             height,
+					   EvRenderFlags    flags);
+void     ev_job_render_set_selection_info (EvJobRender     *job,
 					   EvRectangle     *selection_points,
+					   EvSelectionStyle selection_style,
 					   GdkColor        *text,
-					   GdkColor        *base,
-					   gboolean         include_forms,
-					   gboolean         include_links,
-					   gboolean         include_images,
-					   gboolean         include_text,
-					   gboolean         include_selection);
-void            ev_job_render_run         (EvJobRender     *thumbnail);
-
+					   GdkColor        *base);
 /* EvJobThumbnail */
 GType           ev_job_thumbnail_get_type (void) G_GNUC_CONST;
 EvJob          *ev_job_thumbnail_new      (EvDocument      *document,
-					   EvRenderContext *rc);
-void            ev_job_thumbnail_run      (EvJobThumbnail  *thumbnail);
-
+					   gint             page,
+					   gint             rotation,
+					   gdouble          scale);
 /* EvJobFonts */
 GType 		ev_job_fonts_get_type 	  (void) G_GNUC_CONST;
 EvJob 	       *ev_job_fonts_new 	  (EvDocument      *document);
-void		ev_job_fonts_run 	  (EvJobFonts 	   *fonts);
 
 /* EvJobLoad */
 GType 		ev_job_load_get_type 	  (void) G_GNUC_CONST;
 EvJob 	       *ev_job_load_new 	  (const gchar 	   *uri,
 					   EvLinkDest      *dest,
-					   EvWindowRunMode  mode);
+					   EvWindowRunMode  mode,
+					   const gchar     *search_string);
 void            ev_job_load_set_uri       (EvJobLoad       *load,
 					   const gchar     *uri);
-void		ev_job_load_run 	  (EvJobLoad 	   *load);
+void            ev_job_load_set_password  (EvJobLoad       *job,
+					   const gchar     *password);
+
+/* EvJobSave */
+GType           ev_job_save_get_type      (void) G_GNUC_CONST;
+EvJob          *ev_job_save_new           (EvDocument      *document,
+					   const gchar     *uri,
+					   const gchar     *document_uri);
 
 /* EvJobPrint */
 GType           ev_job_print_get_type     (void) G_GNUC_CONST;
@@ -269,7 +346,6 @@ EvJob          *ev_job_print_new          (EvDocument      *document,
 					   gint             copies,
 					   gdouble          collate,
 					   gdouble          reverse);
-void            ev_job_print_run          (EvJobPrint      *print);
 
 G_END_DECLS
 
